--[[	QuickFrames_Interrupts - A lightweight interrupt tracking module!

Purpose:	Provides interrupt tracking and communication to other raid members to allow
			for more effecient usage of interrupts for raid content. Can be ran independantly
			without a display to allow other members to track your interrupts or with
			QuickFrames to give a display.
]]
 
require "Window"
 
-----------------------------------------------------------------------------------------------
-- QuickFrames_Interrupts Module Definition
-----------------------------------------------------------------------------------------------
local QuickFrames_Interrupts = { } 
 
-----------------------------------------------------------------------------------------------
-- Constants
-----------------------------------------------------------------------------------------------
local kDebug = false
local ktInterruptsDB = {	-- Taken from Accountability by Allie
	-- Warriors
	[18363] = {
		name = "Grapple",
		iaTiers = {
			1, 1, 1, 1, 2, 2, 2, 2, 2
		}
	},
	
	[18547] = {
		name = "Flash Bang",
		iaTiers = {
			1, 1, 1, 1, 1, 1, 1, 1, 1
		}	
	},
	
	[38017] = {
		name = "Kick",
		iaTiers = {
			1, 1, 1, 1, 1, 1, 1, 1, 2
		}
	},	
	
	-- Engineer
	[25635] = {
		name = "Zap",
		iaTiers = {
			1, 1, 1, 1, 2, 2, 2, 2, 2
		}
	},
	
	[34176] = {
		name = "Obstruct Vision",
		iaTiers = {
			1, 1, 1, 1, 1, 1, 1, 1, 1
		}
	},
	
	-- Esper
	[19022] = {
		name = "Crush",
		iaTiers = {
			1, 1, 1, 1, 2, 2, 2, 2, 2
		}
	},
	
	[19355] = {
		name = "Incapacitate",
		iaTiers = {
			1, 1, 1, 1, 1, 1, 1, 1, 1
		}
	},
	
	[19029] = {
		name = "Shockwave",
		iaTiers = {
			1, 1, 1, 1, 1, 1, 1, 1, 1
		}
	},
	
	-- Medic
	[26543] = {
		name = "Paralytic Surge",
		iaTiers = {
			1, 1, 1, 1, 2, 2, 2, 2, 2
		}
	},
	
	-- Stalker
	[23173] = {
		name = "Stagger",
		iaTiers = {
			1, 1, 1, 1, 1, 1, 1, 1, 1
		}
	},
	
	[23587] = {
		name = "False Retreat",
		sprite = "Icon_SkillShadow_UI_stlkr_shadowdash",
		iaTiers = {
			1, 1, 1, 1, 1, 1, 1, 1, 1
		}
	},
	
	[23705] = {
		name = "Collapse",
		iaTiers = {
			1, 1, 1, 1, 1, 1, 1, 1, 1
		}
	},
	
	-- Spellslinger
	[30160] = {
		name = "Arcane Shock",
		iaTiers = {
			1, 1, 1, 1, 1, 1, 1, 1, 1
		}
	},

	[16454] = {
		name = "Spatial Shift",
		iaTiers = {
			1, 1, 1, 1, 1, 1, 1, 1, 1
		}
	},
	
	[20325] = {
		name = "Gate",
		iaTiers = {
			1, 1, 1, 1, 2, 2, 2, 2, 2
		}
	},
	
}
 
-----------------------------------------------------------------------------------------------
-- Initialization
-----------------------------------------------------------------------------------------------
function QuickFrames_Interrupts:new(o)
    o = o or {}
    setmetatable(o, self)
    self.__index = self

    self.QuickFrames = Apollo.GetAddon("QuickFrames")
    self.tGroup = { }
    self.tMyInterrupts = { }
    self.strPartyLeader = nil
    self.tChannel = nil
    self.bLASUpdate = false
    self.bChannelJoined = false
    self.myName = nil
    self.tOnCooldown = { }

    return o
end

function QuickFrames_Interrupts:Init()
    Apollo.RegisterAddon(self, false, "", { })
end

function QuickFrames_Interrupts:OnLoad()
	self.xmlDoc = XmlDoc.CreateFromFile("QuickFrames_Interrupts.xml")
	self.xmlDoc:RegisterCallback("OnDocLoaded", self)
end

-----------------------------------------------------------------------------------------------
-- QuickFrames_Interrupts OnDocLoaded
-----------------------------------------------------------------------------------------------
function QuickFrames_Interrupts:OnDocLoaded()
	-- Since we don't actually need the form
	self.xmlDoc = nil
	
	-- Register Timers
	self.updateTimer = ApolloTimer.Create(1.0, true, "OnSecTimer", self)
	self.joinChannelTimer = ApolloTimer.Create(0.5, true, "CheckJoinChannel", self)
	self.updateTimer:Stop() 		-- Timers start automatically and we don't wanna update if 
	self.joinChannelTimer:Stop()	-- we're not in a group.

	if GroupLib.InGroup() then
		self:OnGroupJoin()
	end

	-- Register Events
	Apollo.RegisterEventHandler("Group_Join", "OnGroupJoin", self)
	Apollo.RegisterEventHandler("Group_Left", "OnGroupLeft", self)
	Apollo.RegisterEventHandler("Group_Add", "OnGroupAdd", self)
	Apollo.RegisterEventHandler("Group_Remove", "OnGroupRemove", self)
	Apollo.RegisterEventHandler("Group_MemberPromoted", "OnGroupMemberPromoted", self)
	Apollo.RegisterEventHandler("Group_FlagsChanged", "OnGroupFlagsChanged", self)	
	Apollo.RegisterEventHandler("AbilityBookChange", "OnAbilityBookChange", self)
end

-----------------------------------------------------------------------------------------------
-- QuickFrames_Interrupts Functions
-----------------------------------------------------------------------------------------------
function QuickFrames_Interrupts:InitializeGroup()
	-- Set myself
	self.myName = GameLib.GetPlayerUnit():GetName()
	-- Update my own Interrupt table
	self:UpdateMyInterrupts()
	-- Iterate over entire party populating tGroup table and finding the leader
	self:DebugPrint("Joined Group! Creating self.tGroup and attempting to join channel!")
	for i = 1, GroupLib.GetMemberCount() do
		local tPlayer = GroupLib.GetGroupMember(i)

		if not self.tGroup[tPlayer.strCharacterName] then
			self.tGroup[tPlayer.strCharacterName] = -1
		end

		if tPlayer.bIsLeader then
			self.strPartyLeader = tPlayer.strCharacterName
		end
	end
	self.tChannel = ICCommLib.JoinChannel("QFI_" .. self.strPartyLeader, "OnChannelMessage", self)
	self.joinChannelTimer:Start()
	self.updateTimer:Start()
end

function QuickFrames_Interrupts:SendUpdate()
	-- Build Message Table
	self:UpdateMyTotalIABreaks()
	local tMessage = {
		nIABreaks = self.tGroup[self.myName],
		strType = "Update"
	}
	self:DebugPrint("Sending Update Message")
	self.tChannel:SendMessage(tMessage)
	self:UpdateQuickFrames()
end

function QuickFrames_Interrupts:UpdateMember(tMsg, strSender)
	self.tGroup[strSender] = tMsg.nIABreaks
	self:DebugPrint("Updating self.tGroup member!")
	self:UpdateQuickFrames()
end

function QuickFrames_Interrupts:UpdateMyInterrupts()
	self.tMyInterrupts = { }
	local tMyAbilities = AbilityBook.GetAbilitiesList()
	local tMyLAS = ActionSetLib.GetCurrentActionSet()

	if not tMyLAS then return end -- Don't go any further if API Call didn't complete
	-- Iterate over entire LAS
	for k, curAbility in ipairs(tMyLAS) do
		-- If curAbility is an interrupt
		if ktInterruptsDB[curAbility] then
			local abilityData = nil

			-- Get Tier Data
			for l, tData in ipairs(tMyAbilities) do
				if tData.nId == curAbility then
					abilityData = tData.tTiers[tData.nCurrentTier].splObject
					break
				end
			end

			-- Record the Interrupt
			if abilityData then
				self.tMyInterrupts[curAbility] = abilityData
			end
		end
	end
	self:DebugPrint("Scanned LAS! Check Rover for Table!")
end

function QuickFrames_Interrupts:UpdateMyTotalIABreaks()
	self.tGroup[self.myName] = 0
	for k, curInterrupt in pairs(self.tMyInterrupts) do
		if curInterrupt:GetCooldownRemaining() == 0 then 
			self.tGroup[self.myName] = self.tGroup[self.myName] + ktInterruptsDB[k].iaTiers[curInterrupt:GetTier()]
		end
	end
	self:DebugPrint("Updating my Total IA! Set to " .. self.tGroup[self.myName] .. "!")
end

function QuickFrames_Interrupts:CheckForLASUpdate()
	if self.bLASUpdate then
		self.bLASUpdate = false
		self:DebugPrint("LAS Update Flag set! Updating LAS!")
		self:UpdateMyInterrupts()
		self:SendUpdate()
	end
end

function QuickFrames_Interrupts:CheckMyCooldowns()
	-- Check for new cooldowns and record
	for abilityId, curAbility in pairs(self.tMyInterrupts) do
		if curAbility:GetCooldownRemaining() > 0 then
			if not self.tOnCooldown[abilityId] then
				self.tOnCooldown[abilityId] = curAbility
				self:DebugPrint("Cooldown detected, sending update!")
				self:SendUpdate()
			end
		end
	end
	-- Check for any cooldowns that came up
	for abilityId, curAbility in pairs(self.tOnCooldown) do
		if curAbility:GetCooldownRemaining() == 0 then
			self.tOnCooldown[abilityId] = nil
			self:SendUpdate()
		end
	end
end

function QuickFrames_Interrupts:UpdateQuickFrames()
	self.QuickFrames = Apollo.GetAddon("QuickFrames")
	if not self.QuickFrames then return end
	if not self.QuickFrames.bShowInterrupts then return end
	self:DebugPrint("QuickFrames is loaded, updating.")
	-- Ensure we have the most recent version of the RaidFrames
	for memberName, raidMember in pairs(self.QuickFrames.tRaidMembers) do
		-- Do we have interrupt data?
		if self.tGroup[memberName] == -1 then
			raidMember.wndCurrInts:Show(false, true)
		else
			-- Ensure we're showing the Text
			raidMember.wndCurrInts:Show(true, true)
			if self.tGroup[memberName] > 0 then
				raidMember.wndCurrInts:SetTextColor(CColor.new(0, 1, 0))
			else raidMember.wndCurrInts:SetTextColor(CColor.new(1, 0, 0)) end
			raidMember.wndCurrInts:SetText("[" .. self.tGroup[memberName] .. "]")
		end
	end
end

-----------------------------------------------------------------------------------------------
-- QuickFrames_Interrupts Timers
-----------------------------------------------------------------------------------------------
function QuickFrames_Interrupts:OnSecTimer()
	self:CheckForLASUpdate()
	self:CheckMyCooldowns()
end

function QuickFrames_Interrupts:CheckJoinChannel()
	if self.bChannelJoined then return end
	if self.tChannel and self.tChannel:SendMessage{strType = "Join"} then
		self:DebugPrint("Successfully joined channel!")
		self.bChannelJoined = true
		self:SendUpdate()
		self.joinChannelTimer:Stop()
	end
end

-----------------------------------------------------------------------------------------------
-- QuickFrames_Interrupts Events
-----------------------------------------------------------------------------------------------
function QuickFrames_Interrupts:OnGroupJoin()
	if GroupLib.InRaid() then
		self:InitializeGroup()
	end
end

function QuickFrames_Interrupts:OnGroupLeft()
	-- Stop all timers, clear all variables
	self:DebugPrint("Left group! Stopping timer + setting tables to nil!")
	self.updateTimer:Stop()
	self.tGroup = { }
	self.strPartyLeader = nil
	self.tChannel = nil
	self.bChannelJoined = false
end

function QuickFrames_Interrupts:OnGroupAdd(strNewGroupMember)
	self:DebugPrint(strNewGroupMember .. " joined the group! Adding self.tGroup table entry!")
	if not self.tGroup[strNewGroupMember] then
		self.tGroup[strNewGroupMember] = -1
	end
	self:UpdateQuickFrames()
end

function QuickFrames_Interrupts:OnGroupRemove(strPlayerName)
	self:DebugPrint(strPlayerName .. " left the group! Removing self.tGroup table entry!")
	self.tGroup[strPlayerName] = nil
end

function QuickFrames_Interrupts:OnGroupMemberPromoted()
	-- Easiest way to deal with this is just to act as if we left and joined another group
	self:DebugPrint("Member promoted, leaving and rejoining group!")
	self:OnGroupLeft()
	self:OnGroupJoin()
end

function QuickFrames_Interrupts:OnGroupFlagsChanged()
	self:DebugPrint("Check to see if group changed to raid group")
	if GroupLib.InRaid() then
		self:InitializeGroup()
	end
end

function QuickFrames_Interrupts:OnAbilityBookChange()
	-- Carbine Wonkiness requires flag usage here.
	self:DebugPrint("LAS Change Detected! Setting LAS Update flag")
	self.bLASUpdate = true
end

function QuickFrames_Interrupts:OnChannelMessage(channel, tMsg, strSender)
	-- Ensure that we have an entry for the sender
	if self.tGroup[strSender] then
		if tMsg.strType == "Join" then
			self:SendUpdate()
		else
			self:UpdateMember(tMsg, strSender)
			self:DebugPrint("Update message received! Updating self.tGroup Table!")
		end
	end
end
-----------------------------------------------------------------------------------------------
-- QuickFrames_Interrupts Debugging
-----------------------------------------------------------------------------------------------
function QuickFrames_Interrupts:DebugPrint(str)
	if kDebug and str then
		Print("[QFI] " .. str)
	end
end

function QuickFrames_Interrupts:SendToRover(tVariable)
	if kDebug and tVariable then
		SendVarToRover("QFI_Table", tVariable, 0)
	end
end

-----------------------------------------------------------------------------------------------
-- QuickFrames_Interrupts Instance
-----------------------------------------------------------------------------------------------
local QuickFrames_InterruptsInst = QuickFrames_Interrupts:new()
QuickFrames_InterruptsInst:Init()
